@@ -1,10 +1,10 @@
-'''
+"""
 LDTP v2 Core Table.
 
 @author: Eitan Isaacson <eitan@ascender.com>
 @author: Nagappan Alagappan <nagappan@gmail.com>
 @copyright: Copyright (c) 2009 Eitan Isaacson
-@copyright: Copyright (c) 2009 Nagappan Alagappan
+@copyright: Copyright (c) 2009-11 Nagappan Alagappan
 @license: LGPL
 
 http://ldtp.freedesktop.org
@@ -14,10 +14,10 @@ Public License version 2 as published by the Free Software Foundation. This file
 is distributed without any warranty; without even the implied warranty of 
 merchantability or fitness for a particular purpose.
 
-See "COPYING" in the source distribution for more information.
+See 'COPYING' in the source distribution for more information.
 
 Headers in this file shall remain intact.
-'''
+"""
 import re
 import pyatspi 
 from utils import Utils
@@ -25,7 +25,7 @@ from server_exception import LdtpServerException
 
 class Table(Utils):
     def getrowcount(self, window_name, object_name):
-        '''
+        """
         Get count of rows in table object.
         
         @param window_name: Window name to look for, either full name,
@@ -37,7 +37,7 @@ class Table(Utils):
 
         @return: Number of rows.
         @rtype: integer
-        '''
+        """
         obj = self._get_object(window_name, object_name)
 
         try:
@@ -48,7 +48,7 @@ class Table(Utils):
         return itable.nRows
 
     def selectrow(self, window_name, object_name, row_text):
-        '''
+        """
         Select row
         
         @param window_name: Window name to type in, either full name,
@@ -62,7 +62,7 @@ class Table(Utils):
 
         @return: 1 on success.
         @rtype: integer
-        '''
+        """
         obj = self._get_object(window_name, object_name)
 
         try:
@@ -88,20 +88,17 @@ class Table(Utils):
                         for child in children:
                             if self._match_name_to_acc(row_text, child):
                                 self._grab_focus(child)
-                                cell.unref()
                                 return 1
                     finally:
                         if not flag:
                             self._handle_table_cell = False
                 elif self._match_name_to_acc(row_text, cell):
                         self._grab_focus(cell)
-                        cell.unref()
                         return 1
-                cell.unref()
         raise LdtpServerException('Unable to select row: %s' % row_text)
 
     def selectrowpartialmatch(self, window_name, object_name, row_text):
-        '''
+        """
         Select row partial match
         
         @param window_name: Window name to type in, either full name,
@@ -115,7 +112,7 @@ class Table(Utils):
 
         @return: 1 on success.
         @rtype: integer
-        '''
+        """
         obj = self._get_object(window_name, object_name)
 
         try:
@@ -141,20 +138,17 @@ class Table(Utils):
                         for child in children:
                             if re.search(row_text, child.name):
                                 self._grab_focus(child)
-                                cell.unref()
                                 return 1
                     finally:
                         if not flag:
                             self._handle_table_cell = False
                 elif self._match_name_to_acc(row_text, cell):
                         self._grab_focus(cell)
-                        cell.unref()
                         return 1
-                cell.unref()
         raise LdtpServerException('Unable to select row: %s' % row_text)
 
     def selectrowindex(self, window_name, object_name, row_index):
-        '''
+        """
         Select row index
         
         @param window_name: Window name to type in, either full name,
@@ -168,7 +162,7 @@ class Table(Utils):
 
         @return: 1 on success.
         @rtype: integer
-        '''
+        """
         obj = self._get_object(window_name, object_name)
 
         try:
@@ -181,11 +175,10 @@ class Table(Utils):
 
         cell = tablei.getAccessibleAt(row_index, 0)
         self._grab_focus(cell)
-        cell.unref()
         return 1
 
     def selectlastrow(self, window_name, object_name):
-        '''
+        """
         Select last row
         
         @param window_name: Window name to type in, either full name,
@@ -197,7 +190,7 @@ class Table(Utils):
 
         @return: 1 on success.
         @rtype: integer
-        '''
+        """
         obj = self._get_object(window_name, object_name)
 
         try:
@@ -207,11 +200,95 @@ class Table(Utils):
 
         cell = tablei.getAccessibleAt(tablei.nRows - 1, 0)
         self._grab_focus(cell)
-        cell.unref()
         return 1
 
+    def setcellvalue(self, window_name, object_name, row_index,
+                     column = 0, data = None):
+        """
+        Set cell value
+        
+        @param window_name: Window name to type in, either full name,
+        LDTP's name convention, or a Unix glob.
+        @type window_name: string
+        @param object_name: Object name to type in, either full name,
+        LDTP's name convention, or a Unix glob. 
+        @type object_name: string
+        @param row_index: Row index to get
+        @type row_index: index
+        @param column: Column index to get, default value 0
+        @type column: index
+        @param data: data, default value None
+                None, used for toggle button
+        @type data: string
+
+        @return: 1 on success.
+        @rtype: integer
+        """
+        obj = self._get_object(window_name, object_name)
+
+        cell = self._get_accessible_at_row_column(obj, row_index, column)
+        print cell
+        name = None
+        if cell.childCount > 0:
+            flag = False
+            try:
+                if self._handle_table_cell:
+                    # Was externally set, let us not
+                    # touch this value
+                    flag = True
+                else:
+                    self._handle_table_cell = True
+                children = self._list_objects(cell)
+                for child in children:
+                    try:
+                        iaction = child.queryAction()
+                    except NotImplementedError:
+                        iaction = None
+                    if iaction:
+                        for i in xrange(iaction.nActions):
+                            # If the cell is toggle type
+                            if re.match('toggle', iaction.getName(i), re.I):
+                                iaction.doAction(i)
+                                self._grab_focus(child)
+                                return 1
+                            elif re.match('edit', iaction.getName(i), re.I):
+                                try:
+                                    texti = cell.queryEditableText()
+                                except NotImplementedError:
+                                    continue
+                                self._grab_focus(child)
+                                if not data:
+                                    raise LdtpServerException('data cannot be empty string.')
+                                return int(texti.setTextContents(data.encode('utf-8')))
+                else:
+                    raise LdtpServerException('Text cannot be entered into object.')
+            finally:
+                if not flag:
+                    self._handle_table_cell = False
+        else:
+            try:
+                iaction = cell.queryAction()
+            except NotImplementedError:
+                iaction = None
+            self._grab_focus(cell)
+            if iaction:
+                for i in xrange(iaction.nActions):
+                    # If the cell is toggle type
+                    if re.match('toggle', iaction.getName(i), re.I):
+                        iaction.doAction(i)
+                        return 1
+                    elif re.match('edit', iaction.getName(i), re.I):
+                        try:
+                            texti = cell.queryEditableText()
+                        except NotImplementedError:
+                            raise LdtpServerException('Text cannot be entered into object.')
+                        if not data:
+                            raise LdtpServerException('data cannot be empty string.')
+                        return int(texti.setTextContents(data.encode('utf-8')))
+        raise LdtpServerException('Text cannot be entered into object.')
+
     def getcellvalue(self, window_name, object_name, row_index, column = 0):
-        '''
+        """
         Get cell value
         
         @param window_name: Window name to type in, either full name,
@@ -227,7 +304,7 @@ class Table(Utils):
 
         @return: cell value on success.
         @rtype: string
-        '''
+        """
         obj = self._get_object(window_name, object_name)
 
         cell = self._get_accessible_at_row_column(obj, row_index, column)
@@ -256,13 +333,12 @@ class Table(Utils):
         else:
             name = cell.name
             self._grab_focus(cell)
-        cell.unref()
         if not name:
             raise LdtpServerException('Unable to get row text')
         return name
 
     def checkrow(self, window_name, object_name, row_index, column = 0):
-        '''
+        """
         Check row
         
         @param window_name: Window name to type in, either full name,
@@ -278,7 +354,7 @@ class Table(Utils):
 
         @return: cell value on success.
         @rtype: string
-        '''
+        """
         obj = self._get_object(window_name, object_name)
 
         cell = self._get_accessible_at_row_column(obj, row_index, column)
@@ -315,13 +391,12 @@ class Table(Utils):
             except NotImplementedError:
                 raise LdtpServerException('Unable to check row')
             self._grab_focus(cell)
-        cell.unref()
         if not flag:
             raise LdtpServerException('Unable to check row')
         return 1
 
     def expandtablecell(self, window_name, object_name, row_index, column = 0):
-        '''
+        """
         Expand or contract table cell
         
         @param window_name: Window name to type in, either full name,
@@ -337,7 +412,7 @@ class Table(Utils):
 
         @return: cell value on success.
         @rtype: string
-        '''
+        """
         obj = self._get_object(window_name, object_name)
 
         cell = self._get_accessible_at_row_column(obj, row_index, column)
@@ -372,13 +447,12 @@ class Table(Utils):
             except NotImplementedError:
                 raise LdtpServerException('Unable to check row')
             self._grab_focus(cell)
-        cell.unref()
         if not flag:
             raise LdtpServerException('Unable to check row')
         return 1
 
     def uncheckrow(self, window_name, object_name, row_index, column = 0):
-        '''
+        """
         Check row
         
         @param window_name: Window name to type in, either full name,
@@ -394,7 +468,7 @@ class Table(Utils):
 
         @return: 1 on success.
         @rtype: integer
-        '''
+        """
         obj = self._get_object(window_name, object_name)
 
         cell = self._get_accessible_at_row_column(obj, row_index, column)
@@ -431,13 +505,12 @@ class Table(Utils):
             except NotImplementedError:
                 raise LdtpServerException('Unable to check row')
             self._grab_focus(cell)
-        cell.unref()
         if not flag:
             raise LdtpServerException('Unable to check row')
         return 1
 
     def gettablerowindex(self, window_name, object_name, row_text):
-        '''
+        """
         Get table row index matching given text
         
         @param window_name: Window name to type in, either full name,
@@ -451,7 +524,7 @@ class Table(Utils):
 
         @return: row index matching the text on success.
         @rtype: integer
-        '''
+        """
         obj = self._get_object(window_name, object_name)
 
         try:
@@ -477,20 +550,17 @@ class Table(Utils):
                         for child in children:
                             if self._match_name_to_acc(row_text, child):
                                 self._grab_focus(child)
-                                cell.unref()
                                 return i
                     finally:
                         if not flag:
                             self._handle_table_cell = False
                 elif self._match_name_to_acc(row_text, cell):
                     self._grab_focus(cell)
-                    cell.unref()
                     return i
-                cell.unref()
         raise LdtpServerException('Unable to get row index: %s' % row_text)
 
     def getrowcount(self, window_name, object_name):
-        '''
+        """
         Select row partial match
         
         @param window_name: Window name to type in, either full name,
@@ -502,7 +572,7 @@ class Table(Utils):
 
         @return: Row count on success.
         @rtype: integer
-        '''
+        """
         obj = self._get_object(window_name, object_name)
 
         try:
@@ -513,7 +583,7 @@ class Table(Utils):
         return tablei.nRows
 
     def singleclickrow(self, window_name, object_name, row_text):
-        '''
+        """
         Single click row matching given text
         
         @param window_name: Window name to type in, either full name,
@@ -527,7 +597,7 @@ class Table(Utils):
 
         @return: row index matching the text on success.
         @rtype: integer
-        '''
+        """
         obj = self._get_object(window_name, object_name)
 
         try:
@@ -557,7 +627,6 @@ class Table(Utils):
                                 self._mouse_event(size.x + size.width / 2,
                                                   size.y + size.height / 2,
                                                   'b1c')
-                                cell.unref()
                                 return i
                     finally:
                         if not flag:
@@ -568,13 +637,11 @@ class Table(Utils):
                     self._mouse_event(size.x + size.width / 2,
                                       size.y + size.height / 2,
                                       'b1c')
-                    cell.unref()
                     return i
-                cell.unref()
         raise LdtpServerException('Unable to get row index: %s' % row_text)
 
     def doubleclickrow(self, window_name, object_name, row_text):
-        '''
+        """
         Single click row matching given text
         
         @param window_name: Window name to type in, either full name,
@@ -588,7 +655,7 @@ class Table(Utils):
 
         @return: row index matching the text on success.
         @rtype: integer
-        '''
+        """
         obj = self._get_object(window_name, object_name)
 
         try:
@@ -618,7 +685,6 @@ class Table(Utils):
                                 self._mouse_event(size.x + size.width / 2,
                                                   size.y + size.height / 2,
                                                   'b1d')
-                                cell.unref()
                                 return i
                     finally:
                         if not flag:
@@ -629,14 +695,12 @@ class Table(Utils):
                     self._mouse_event(size.x + size.width / 2,
                                       size.y + size.height / 2,
                                       'b1d')
-                    cell.unref()
                     return i
-                cell.unref()
         raise LdtpServerException('Unable to get row index: %s' % row_text)
 
     def verifytablecell(self, window_name, object_name, row_index,
                         column_index, row_text):
-        '''
+        """
         Verify table cell value with given text
         
         @param window_name: Window name to type in, either full name,
@@ -654,7 +718,7 @@ class Table(Utils):
 
         @return: 1 on success 0 on failure.
         @rtype: integer
-         '''
+         """
         try:
             text = self.getcellvalue(window_name, object_name,
                                      row_index, column_index)
@@ -663,7 +727,7 @@ class Table(Utils):
             return 0
 
     def doesrowexist(self, window_name, object_name, row_text):
-        '''
+        """
         Verify table cell value with given text
         
         @param window_name: Window name to type in, either full name,
@@ -677,7 +741,7 @@ class Table(Utils):
 
         @return: 1 on success 0 on failure.
         @rtype: integer
-        '''
+        """
         try:
             obj = self._get_object(window_name, object_name)
 
@@ -696,7 +760,7 @@ class Table(Utils):
 
     def verifypartialtablecell(self, window_name, object_name, row_index,
                                column_index, row_text):
-        '''
+        """
         Verify partial table cell value
         
         @param window_name: Window name to type in, either full name,
@@ -714,7 +778,7 @@ class Table(Utils):
 
         @return: 1 on success 0 on failure.
         @rtype: integer
-        '''
+        """
         try:
             text = self.getcellvalue(window_name, object_name, row_index, column)
             if re.search(row_text, text):
